<?php

namespace FiloBlu\Flow\Model\Parser;

/**
 * Class Csv
 * @package FiloBlu\Flow\Model\Parser
 */
class Csv extends AbstractParser
{
    /**
     * @var
     */
    public $map;

    /**
     * @var
     */
    public $pointer;

    /**
     * @var array
     */
    public $header;

    /**
     * @var int
     */
    public $check_mapping_count = 1;

    /**
     * @var
     */
    protected $_config;

    /**
     * @var \Monolog\Logger
     */
    protected $_logger;

    /**
     * @var int
     */
    private $linesToSkip = 0;

    /**
     * @var string
     */
    private $separator = ';';

    /**
     * @var string
     */
    private $delimiter = '"';

    /**
     * @var bool
     */
    protected $_isCaseInsensitive = false;
    
    /**
     * Csv constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
    ) {
        $this->_logger = $loggerProvider->getLogger();
        parent::__construct($loggerProvider);
    }

    /**
     * @param $config
     * @return AbstractParser|void
     */
    public function setConfig($config)
    {
        $this->_config = $config;

        $this->separator = (String)@$this->_config['separator'];
        // TODO : in channel is set to $newline but has to be set to "
        // $this->delimiter = (String)@$this->_config['delimiter'];
        $this->linesToSkip = (String)@$this->_config['lines_to_skip'];
        $this->check_mapping_count = (String)@$this->_config['check_mapping_count'];

    }

    /**
     * @param $resource string absolute path of the the csv file
     * @return array|mixed
     * @throws \Exception
     */
    public function process($resource)
    {
        $returnArray = [];
        $count = 0;

        if (false === ($handle = @fopen($resource, 'r'))) {
            throw new \Exception("Unable to open {$resource}");
        }

        while (($data = fgetcsv($handle, 0, $this->separator, $this->delimiter)) !== false) {

            $count++;

            if ($count == 1) {

                if($this->_isCaseInsensitive)
                {
                    foreach ($data as $header)
                    {
                        $this->header[] = strtolower($header);
                    }
                } else{
                    $this->header = $data;
                }
            }

            if ($count <= $this->linesToSkip) {
                continue;
            }

            try {
                $returnArray[] = $this->parse($data);
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage() . ' at row' . $count);
            }

        }

        fclose($handle);

        $channel = $this->getChannel();

        if (method_exists($channel, 'commitData') && !is_null($channel)) {
            $channel->commitData();
        }

        return $returnArray;
    }

}
