<?php
/**
 * Copyright © 2015 FiloBlu. All rights reserved.
 */

namespace FiloBlu\Flow\Setup;

use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\Setup\ModuleContextInterface;
/**
 * @codeCoverageIgnore
 */
class UpgradeSchema implements UpgradeSchemaInterface
{
    /**
     * {@inheritdoc}
     */
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        
        $setup->startSetup();

        if(!$context->getVersion()) {
            
        }
        
        if (version_compare($context->getVersion(), '0.0.8') < 0) {
              $this->createFlowFromStockTable($setup, $context);
        }
        
        if (version_compare($context->getVersion(), '0.1.1') < 0) {
             $this->addImagesTable($setup, $context);
        }
        
        if (version_compare($context->getVersion(), '0.1.2') < 0) {
             $this->addUniqueConstraints($setup, $context);
        }
        
        if (version_compare($context->getVersion(), '0.1.3') < 0) {
             $this->addMetaFileToStock($setup, $context);
        }
                
        if (version_compare($context->getVersion(), '0.1.6') < 0) {
             $this->addProcessImagesTables($setup, $context);
        } 
        
        if (version_compare($context->getVersion(), '0.2.1') < 0) {
             $this->addImagesHideColumn($setup, $context);
        } 
        
        if (version_compare($context->getVersion(), '0.4.5') < 0) {
             $this->addUniqueConstraints2($setup, $context);
        }

        if (version_compare($context->getVersion(), '1.0.0') < 0) {
            $this->addColumnsToIte($setup, $context);
        }

        if (version_compare($context->getVersion(), '1.0.1') < 0) {
            $this->addVisibilityToIte($setup, $context);
        }

        if (version_compare($context->getVersion(), '1.0.2') < 0) {
            $this->changeCodConfigurableLength($setup, $context);
        }

        if (version_compare($context->getVersion(), '1.0.3') < 0) {
            $this->addExecutedOnField($setup, $context);
        }

        if (version_compare($context->getVersion(), '1.0.4') < 0) {
            $this->addLanguageFieldOnEavs($setup, $context);
        }

        if (version_compare($context->getVersion(), '1.0.5') < 0) {
            $this->addAttributeSetOnIte($setup, $context);
        }

        if (version_compare($context->getVersion(), '1.0.6') < 0) {
            $this->changeFasciaIvaArticoliLength($setup, $context);
        }
        
        $setup->endSetup();

    }
        
     public function createFlowFromStockTable(SchemaSetupInterface $setup, ModuleContextInterface $context)
        {
         
          /**
           * Create table 'flow_from_stock' if not exists
           */

          if (!$setup->tableExists('flow_from_stock')) {

              $table = $setup->getConnection()->newTable(
                  $setup->getTable('flow_from_stock')
              )
                  ->addColumn(
                        'sku',
                        \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        '40',
                        [],
                        'sku'
                  )
                  ->addColumn(
                        'magazzino',
                        \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        '30',
                        [],
                        'magazzino'
                  )
                  ->addColumn(
                        'fisico_disponibile',
                        \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        '10',
                        [],
                        'qta disponibile in magazzino - aggiornamento in sovrascrittura'
                  )
                  ->addColumn(
                        'meta_id',
                        \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        null,
                        ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                        'id'
                    )
                  ->addColumn(
                      'meta_ref_id',
                      \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                      '11',
                      [],
                      'meta_ref_id'
                  )
                  ->addColumn(
                        'meta_processed',
                        \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        '1',
                        [],
                        'meta_processed'
                    )
                  ->addColumn(
                        'meta_process_time',
                        \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                        '10',
                        [],
                        'meta_process_time'
                    )

                  ->setComment(
                      'FiloBlu Flow flow_from_stock'
                  );

              $setup->getConnection()->createTable($table);


          }

         
        }

        public function addImagesTable ($setup, $context) {

        /**
         * Create table 'filoblu_imageflow_from' if not exists
         */

        if (!$setup->tableExists('flow_imagesflow')) {

            $table = $setup->getConnection()->newTable(
                $setup->getTable('flow_imagesflow')
            )
                ->addColumn(
                    'id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                    null
                )
                ->addColumn(
                    'image_name',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    null
                )
                ->addColumn(
                    'image_status',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '30',
                    [],
                    null
                )
                ->addColumn(
                    'image_queue_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    null,
                    [],
                    null
                )
                ->addColumn(
                    'image_receive_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    null,
                    [],
                    null
                )
                ->addColumn(
                    'image_process_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    null,
                    [],
                    null
                )        
                ->addColumn(
                    'meta_file',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['nullable' => false],
                    null
                )                       
                ->addColumn(
                    'meta_processed',
                    \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                    "1",
                    ['nullable' => false, 'default' => '0'],
                    null
                )   
                ->addColumn(
                    'log',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    null
                );                    

                /*{{CedAddTableColumn}}}*/

            $setup->getConnection()->createTable($table);
            /*{{CedAddTable}}*/


        }
      }
      
      public function addUniqueConstraints(SchemaSetupInterface $setup, ModuleContextInterface $context)
        {
            try{
                $setup->getConnection()->query("ALTER TABLE `flow_inboundflow` ADD UNIQUE (`name`(255));");
                $setup->getConnection()->query("ALTER TABLE `flow_channels` ADD UNIQUE (`name`(40));");
            } catch(\Exception $e) {
                throw new \Exception("Error Processing Request - add unique constraints on `flow_inboundflow` and `flow_channels` " . $e->getMessage() , 1);
            }
        }
        
      public function addMetaFileToStock(SchemaSetupInterface $setup, ModuleContextInterface $context) {
            
            if ($setup->tableExists('flow_from_stock')) {
                
                $setup->getConnection()
                        
                ->addColumn(
                        $setup->getTable('flow_from_stock'),
                        'meta_file',
                        [
                            'type' => \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                            'nullable' => false,
                            'comment' => 'meta_file',
                            'after' => 'meta_id'
                        ]
                        
                 );    
                
            }
            
      }

    public function addExecutedOnField(SchemaSetupInterface $setup, ModuleContextInterface $context) {

        if ($setup->tableExists('flow_from_eavs')) {

            $setup->getConnection()

                ->addColumn(
                    $setup->getTable('flow_from_eavs'),
                    'executed_on',
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => '25',
                        'nullable' => true,
                        'comment' => 'executed_on',
                        'after' => 'meta_ref_id'
                    ]

                );

        }

    }

      
      public function addProcessImagesTables(SchemaSetupInterface $setup, ModuleContextInterface $context) {
        if ($setup->tableExists('flow_imagesflow')) {

            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable('flow_imagesflow'), 'image_sku', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => '100',
                        'nullable' => false,
                        'comment' => 'image_sku',
                        'after' => 'image_name'
                            ]
            );
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable('flow_imagesflow'), 'image_position', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => '10',
                        'nullable' => false,
                        'comment' => 'image_position',
                        'after' => 'image_name'
                            ]
            );
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable('flow_imagesflow'), 'image_path', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => '64k',
                        'nullable' => false,
                        'comment' => 'image_path',
                        'after' => 'image_name'
                            ]
            );  
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable('flow_imagesflow'), 'image_alt', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => '255',
                        'nullable' => true,
                        'comment' => 'image_alt',
                        'after' => 'image_name'
                            ]
            ); 
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable('flow_imagesflow'), 'image_type', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => '50',
                        'nullable' => false,
                        'comment' => 'image_type',
                        'after' => 'image_name'
                            ]
            );            
        }
    }
    
    public function addImagesHideColumn(SchemaSetupInterface $setup, ModuleContextInterface $context) {
        if ($setup->tableExists('flow_imagesflow')) {
            $setup->getConnection()
                    ->addColumn(
                            $setup->getTable('flow_imagesflow'), 'image_hide', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => '10',
                        'nullable' => false,
                        'comment' => 'image_hide',
                        'after' => 'image_name'
                            ]
            );
        }
    }
    
    public function addUniqueConstraints2(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        try{
            $setup->getConnection()->query("ALTER TABLE `flow_from_eavs` ADD UNIQUE INDEX `IDX_E_A_V_S_METAFILE` (`e`, `a`, `v`(100), `s`, `meta_file`);");
        } catch(\Exception $e) {
            throw new \Exception("Error Processing Request - add unique constraints on `flow_from_eavs` : " . $e->getMessage() , 1);
        }
    }


    public function addColumnsToIte(SchemaSetupInterface $setup, ModuleContextInterface $context){


        if ($setup->tableExists('flow_from_ite')) {

            $setup->getConnection()
                ->addColumn(
                    $setup->getTable('flow_from_ite'), 'configurable_values', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 150,
                        'nullable' => true,
                        'after' => 'codice_configurabile',
                        'comment' => 'Attributes needed to create the configurable'
                    ]
                );

            $setup->getConnection()
                ->addColumn(
                    $setup->getTable('flow_from_ite'), 'configurable_attributes', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 200,
                        'nullable' => true,
                        'after' => 'codice_configurabile',
                        'comment' => 'Attributes needed to create the configurable'
                    ]
                );
        }
    }

    public function addVisibilityToIte(SchemaSetupInterface $setup, ModuleContextInterface $context){


        if ($setup->tableExists('flow_from_ite')) {

            $setup->getConnection()
                ->addColumn(
                    $setup->getTable('flow_from_ite'), 'visibility', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 10,
                        'nullable' => true,
                        'after' => 'fascia_iva_articoli',
                        'comment' => 'Visibility of the product, not mandatory'
                    ]
                );
        }
    }

    public function addAttributeSetOnIte(SchemaSetupInterface $setup, ModuleContextInterface $context){


        if ($setup->tableExists('flow_from_ite')) {

            $setup->getConnection()
                ->addColumn(
                    $setup->getTable('flow_from_ite'), 'attribute_set', [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 25,
                        'nullable' => true,
                        'after' => 'visibility',
                        'comment' => 'Attribute Set of the product, not mandatory'
                    ]
                );
        }
    }


    public function changeCodConfigurableLength(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        try {
            $setup->getConnection()->query("ALTER TABLE `flow_from_ite` CHANGE COLUMN `codice_configurabile` `codice_configurabile` VARCHAR(255) NULL DEFAULT NULL COMMENT 'codice_configurabile'");
        } catch(\Exception $e) {
            throw new \Exception("Error Altering table `flow_from_ite` : " . $e->getMessage() , 1);
        }
    }

    public function addLanguageFieldOnEavs(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        try {
            $setup->getConnection()->query("ALTER TABLE `flow_from_eavs` ADD COLUMN `l` VARCHAR(20) NULL DEFAULT NULL COMMENT 'l' AFTER `s`");
        } catch(\Exception $e) {
            throw new \Exception("Error Altering table `flow_from_eavs` : " . $e->getMessage() , 1);
        }
    }

    public function changeFasciaIvaArticoliLength(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        try {
            $setup->getConnection()->query("ALTER TABLE `flow_from_ite` CHANGE COLUMN `fascia_iva_articoli` `fascia_iva_articoli` VARCHAR(100) NULL DEFAULT NULL COMMENT 'fascia_iva_articoli'");
        } catch(\Exception $e) {
            throw new \Exception("Error Altering table `flow_from_ite` : " . $e->getMessage() , 1);
        }
    }
      
}
